@@ -329,6 +329,25 @@ static const char unknown[8] = "obsolete";
 //static const char unknown[8] = "unknown";
 #define unknown "unknown-"
 
+static const char *feat_word69_str[16] = { 
+	"CFast specification support",			/* word 69 bit 15 */
+	"Deterministic read data after TRIM",		/* word 69 bit 14 */
+	"Long physical sector diagnostics",		/* word 69 bit 13 */
+	"DEVICE CONFIGURATION SET/IDENTIFY DMA commands", /* word 69 bit 12 */
+	"READ BUFFER DMA command",			/* word 69 bit 11 */
+	"WRITE BUFFER DMA command",			/* word 69 bit 10 */
+	"SET MAX SETPASSWORD/UNLOCK DMA commands",	/* word 69 bit  9 */
+	"DOWNLOAD MICROCODE DMA command",		/* word 69 bit  8 */
+	"reserved 69[7]",				/* word 69 bit  7 */
+	"reserved 69[6]",				/* word 69 bit  6 */
+	"Deterministic read ZEROs after TRIM",		/* word 69 bit  5 */
+	"reserved 69[4]",				/* word 69 bit  4 */
+	"reserved 69[3]",				/* word 69 bit  3 */
+	"reserved 69[2]",				/* word 69 bit  2 */
+	"reserved 69[1]",				/* word 69 bit  1 */
+	"reserved 69[0]",				/* word 69 bit  0 */
+};
+
 static const char *feat_word82_str[16] = { 
 	"obsolete 82[15]",				/* word 82 bit 15: obsolete  */
 	"NOP cmd",					/* word 82 bit 14 */
@@ -414,7 +433,7 @@ static const char *cap_sata0_str[16] = {
 	"unknown 76[6]",				/* word 76 bit  6 */
 	"unknown 76[5]",				/* word 76 bit  5 */
 	"unknown 76[4]",				/* word 76 bit  4 */
-	"unknown 76[3]",				/* word 76 bit  3 */
+	"Gen3 signaling speed (6.0Gb/s)",		/* word 76 bit  3 */
 	"Gen2 signaling speed (3.0Gb/s)",		/* word 76 bit  2 */
 	"Gen1 signaling speed (1.5Gb/s)",		/* word 76 bit  1 */
 	"unknown 76[0]"					/* word 76 bit  0 */
@@ -919,7 +938,7 @@ void identify (__u16 *id_supplied)
 			printf("\t%-31s %11u bytes\n","Physical Sector size:", sector_bytes * pfactor);
 			if ((val[209] & 0xc000) == 0x4000) {
 				unsigned int offset = val[209] & 0x1fff;
-				printf("\t%-31s %11u bytes\n", "Logical Sector-0 offset:", offset * lsize);
+				printf("\t%-31s %11u bytes\n", "Logical Sector-0 offset:", offset * sector_bytes);
 			}
 		}
 		if (!bbbig) bbbig = (__u64)(ll>mm ? ll : mm); /* # 512 byte blocks */
@@ -1082,7 +1101,8 @@ void identify (__u16 *id_supplied)
 		sdma_ok = 0;  /* word 62 has been re-purposed for non-sdma mode reporting */
 		printf("\tDMADIR bit required in PACKET commands\n");
 	} else {
-		__u8 w62 = val[62], hi = w62 >> 8, lo = w62;
+		__u16 w62 = val[62];
+		__u8 hi = w62 >> 8, lo = w62;
 		if (!w62 || (lo & 0xf8))
 			sdma_ok = 0;
 		else if (hi && hi != 1 && hi != 2 && hi != 4)
@@ -1185,14 +1205,21 @@ void identify (__u16 *id_supplied)
 		if (val[SCT_SUPP] & 0x1)
 			print_features(val[SCT_SUPP], val[SCT_SUPP] & 0x3f, feat_sct_str);
 	}
-
-	if (val[169] && val[169] != 0xffff) {
-		if (val[169] & 1) {
-			const char *behaviour = "in";
-			if (val[69] & (1 << 14))
-				behaviour = "";
-			printf("\t   *\tData Set Management %sdeterminate TRIM supported\n", behaviour);
+	if (like_std > 6) {
+		const __u16 trimd = 1<<14;	/* deterministic read data after TRIM */
+		const __u16 trimz = 1<<5;	/* deterministic read ZEROs after TRIM */
+		__u16 word69 = val[69] & ~(trimz | trimd); /* TRIM bits require special interpretation */
+		print_features(word69, word69, feat_word69_str);
+		if (val[169] & 1 && val[169] != 0xffff) { /* supports TRIM ? */
+			printf("\t   *\tData Set Management TRIM supported\n");
+			if (val[69] & trimd) { /* Deterministic TRIM support */
+				if (val[69] & trimz)
+					print_features(trimz, trimz, feat_word69_str);
+				else
+					print_features(trimd, trimd, feat_word69_str);
+			}
 		}
+		
 	}
 
 	if (is_cfa) {
@@ -1304,7 +1331,8 @@ void identify (__u16 *id_supplied)
 			printf("\n");
 		}
 	}
-	if((eqpt != CDROM) && (like_std > 3) && (val[CMDS_EN_2] & WWN_SUP)) {
+	//printf("w84=0x%04x w87=0x%04x like_std=%d\n", val[84], val[87], like_std);
+	if((eqpt != CDROM) && (like_std > 3) && (val[CMDS_SUPP_2] & WWN_SUP)) {
 		printf("Logical Unit WWN Device Identifier: %04x%04x%04x%04x\n", val[108], val[109], val[110], val[111]);
 		printf("\tNAA\t\t: %x\n", (val[108] & 0xf000) >> 12);
 		printf("\tIEEE OUI\t: %06x\n", (((val[108] & 0x0fff) << 12) | ((val[109] & 0xfff0) >> 4)));
